
STANDARD_TRAILER_DIMS = [
    {
        'name': "Standard 53' Dry-Van",
        'code': 'DV_53',
        'inner_width': 98.5,
        'inner_height': 108,
        'inner_length': 52 * 12 + 6,
        'max_weight': 42500,
    },
    {
        'name': "Standard 48' Dry-Van",
        'code': 'DV_48',
        'inner_width': 98.5,
        'inner_height': 100.6,
        'inner_length': 47 * 12 + 6,
        'max_weight': 13500,
    },
    {
        'name': "Standard 45' Dry-Van",
        'code': 'DV_45',
        'inner_width': 93,
        'inner_height': 100.5,
        'inner_length': 44 * 12 + 1.5,
        'max_weight': 11900,
    },
    {
        'name': "Standard 43' Dry-Van",
        'code': 'DV_43',
        'inner_width': 92.5,
        'inner_height': 100.5,
        'inner_length': 41 * 12 + 6,
        'max_weight': 11900,
    },
    {
        'name': "Standard 40' Dry-Van",
        'code': 'DV_40',
        'inner_width': 92.5,
        'inner_height': 94.5,
        'inner_length': 39 * 12 + 6,
        'max_weight': 11300,
    },
    {
        'name': "28' Dry-Van, Pup - Roll Door",
        'code': 'DV_28_RD',
        'inner_width': 100,
        'inner_height': 107,
        'inner_length': 27 * 12 + 3,
        'max_weight': 22000,
    },
    {
        'name': "28' Dry-Van, Pup - Swing Door",
        'code': 'DV_28_SD',
        'inner_width': 99,
        'inner_height': 108,
        'inner_length': 27 * 12 + 3,
        'max_weight': 22000,
    },
    {
        'name': "20' Shipping Container",
        'code': 'CONT_20',
        'inner_width': 7 * 12 + 8,
        'inner_height': 7 * 12 + 9,
        'inner_length': 19 * 12 + 4,
        'max_weight': 22000,
    },
    {
        'name': "40' Shipping Container",
        'code': 'CONT_40',
        'inner_width': 7 * 12 + 8,
        'inner_height': 7 * 12 + 10,
        'inner_length': 39 * 12 + 6,
        'max_weight': 22000,
    },
    {
        'name': "45' Shipping Container",
        'code': 'CONT_45',
        'inner_width': 7 * 12 + 8,
        'inner_height': 8 * 12 + 10,
        'inner_length': 44 * 12 + 6,
        'max_weight': 22000,
    },
    {
        'name': "48' Shipping Container",
        'code': 'CONT_48',
        'inner_width': 8 * 12 + 2,
        'inner_height': 8 * 12 + 10,
        'inner_length': 47 * 12 + 6,
        'max_weight': 22000,
    },
    {
        'name': "53' Shipping Container",
        'code': 'CONT_53',
        'inner_width': 8 * 12 + 2,
        'inner_height': 8 * 12 + 10,
        'inner_length': 52 * 12 + 3,
        'max_weight': 42500,
    },
    {
        'name': "26' Flatbed Trailer",
        'code': 'FB_26',
        'inner_width': 96,
        'inner_height': 102,
        'inner_length': 26 * 12,
        'max_weight': 55000,
    },
    {
        'name': "40' Flatbed Trailer",
        'code': 'FB_40',
        'inner_width': 96,
        'inner_height': 102,
        'inner_length': 40 * 12,
        'max_weight': 55000,
    },
    {
        'name': "45' Flatbed Trailer",
        'code': 'FB_45',
        'inner_width': 102,
        'inner_height': 102,
        'inner_length': 45 * 12,
        'max_weight': 55000,
    },
    {
        'name': "48' Flatbed Trailer",
        'code': 'FB_48',
        'inner_width': 102,
        'inner_height': 102,
        'inner_length': 48 * 12,
        'max_weight': 55000,
    },
    {
        'name': "Small Straight Truck - 22'",
        'code': 'STRGHT_TRK_22',
        'inner_width': 92,
        'inner_height': 8 * 12,
        'inner_length': 21 * 12,
        'max_weight': 14000,
    },
    {
        'name': "Large Straight Truck - 26'",
        'code': 'STRGHT_TRK_26',
        'inner_width': 96,
        'inner_height': 8 * 12,
        'inner_length': 25 * 12,
        'max_weight': 14000,
    },
]
